cccccccc program: adafit.f ccccc version: 17 May 2003 cccccccccccccccccccccccc
c Calculates a best-fit (L1) hyperbola relating the L1 error norm of observed
c and predicted coherence functions to the log of te, and uses the results to
c adaptively guide the choice of elastic thickness estimate in the shellscript.
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c  Modified by Gareth Armstrong,Dept. Earth Sciences,Uni. Oxford,
c  on 17/2/99 to use the Salford 
c  ftn77 compiler call to COMMAND_LINE to read in command line arguments 
c  similar to the f77 compiler call to getarg when being used on the PC
c  Comment or uncomment relevant calls as required.
c  Changed Units numbers to be above 12 
c  Doesn't seem to need to be array size adjustable 
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      implicit none
c
c common variables
c
      real*8 amat(200,200),el1(100),elsk(100),erh(100),te(100),
     &       tesk(100)
      integer ntote
      common /ERRS  / te,el1,erh,ntote
      common /ERSK  / tesk,elsk
      common /MAT   / amat
c
c Local variables
c
      real*8 alt,beste,cl,dmult,efac,rtmn,rtmx,x,y
      real*8 am(3),amp(3),ampp(3)
      character copfile*20,card*40
      real*8 dexp,dfloat,dlog,dsqrt
      integer i,icase,idiff,ik,imn,imx,io,iposv(200),it,
     &        izrov(200),j,mpar
      integer iabs,idnint
c
      call getarg(1,copfile)
c
c ------------------------------------------------------------------
c  SALFORD FTN77 COMPILER COMMAND LINE read SUBROUTINE call SIMILAR
c     TO F77 call TO getarg but for use on PC
c
c  modified by Gareth Armstrong 3/11/96
c-------------------------------------------------------------------
c
c      call COMMand_LINE(card)
c      read(card,*)copfile
c
      open(13,file=copfile,status='old')
      rewind(13)
      open(14,file='tetemp',status='unknown')
      rewind(14)
c
c*****************************************************
c Efac (factor to determine 95% confidence limits)   *
c     was calculated from scaling of errors          *
c     estimated for a synthetic with known T_e.      *
c*****************************************************
      efac=1.42901281596470d0
c
      dmult=2d0**(1d0/12d0)
      do i=1,85
        elsk(i)=0d0
        tesk(i)=dfloat(idnint(20d0*dmult**(i-1)))/10d0
      enddo
c
      mpar=3
      call input
      call evalgt(mpar)
      call simplx(amat,2*ntote,2*mpar+ntote,200,200,ntote,ntote,0,icase,
     &            izrov,iposv)
      do i=1,3
        amp(i)=0d0
        ampp(i)=0d0
      enddo
      do j=1,2*ntote
        if(iposv(j).eq.1)amp(1)=amat(j+1,1)
        if(iposv(j).eq.2)amp(2)=amat(j+1,1)
        if(iposv(j).eq.3)amp(3)=amat(j+1,1)
        if(iposv(j).eq.4)ampp(1)=amat(j+1,1)
        if(iposv(j).eq.5)ampp(2)=amat(j+1,1)
        if(iposv(j).eq.6)ampp(3)=amat(j+1,1)
      enddo
      do i=1,3
        am(i)=ampp(i)-amp(i)
      enddo
      do i=1,ntote
        x=dlog(te(i))
        y=dmax1(am(1)*x*x+am(2)*x+am(3),0d0)
        erh(i)=dsqrt(y)
      enddo
      alt=-am(2)/(2d0*am(1))
      beste=dexp(alt)
      write(*,1005)'Approximate best-fit elastic thickness is ',beste,
     &             ' km.'
      cl=4d0*(am(1)*alt*alt+am(2)*alt+am(3))
      rtmx=dexp((-am(2)+dsqrt(am(2)*am(2)-4d0*am(1)*(am(3)-cl)))
     &       /(2d0*am(1)))
      rtmn=dexp((-am(2)-dsqrt(am(2)*am(2)-4d0*am(1)*(am(3)-cl)))
     &       /(2d0*am(1)))
c
      it=idnint((dlog(beste/2d0)/dlog(dmult))+1d0)
      io=1
      do while(.true.)
        if(elsk(io).eq.0d0)then
          io=io+1
          goto100
        endif
        do i=2,85
          if(elsk(i).lt.elsk(io).and.elsk(i).ne.0d0)io=i
        enddo
        imn=io
        do i=io,1,-1
          if(elsk(i).le.elsk(io)*efac.and.elsk(i).ne.0d0)imn=i
        enddo
        imx=io
        do i=io,85
          if(elsk(i).le.elsk(io)*efac.and.elsk(i).ne.0d0)imx=i
        enddo
        ik=it
        if(ik.ge.84)then
          ik=84
          do while(.true.)
            if(elsk(ik).eq.0d0)goto1
            ik=ik-1
          enddo
1       endif
        if(ik.le.2)then
          ik=2
          do while(.true.)
            if(elsk(ik).eq.0d0)goto2
            ik=ik+1
          enddo
2       endif
        if(elsk(ik).eq.0d0)then
          write(14,'(f5.1)')tesk(ik)
          goto200
        endif
        idiff=100
        do i=max(1,imn-2),min(85,imx+2)
          if(elsk(i).eq.0d0.and.idiff.gt.iabs(io-i))then
            ik=i
            idiff=iabs(io-i)
          endif
        enddo
        write(*,'(a)')' '
        write(*,1005)'True best-fit elastic thickness is ',tesk(io),
     &               ' km.'
        write(*,1005)'Minimum error bound is ',tesk(imn-1),' km.'
        write(*,1005)'Maximum error bound is ',tesk(imx+1),' km.'
        write(*,1010)'Error minimum is ',elsk(io)
        write(*,1015)'(',tesk(io),tesk(imn-1),tesk(imx+1),elsk(io),
     &               ')'
        if(ik.eq.it)write(14,'(f5.1)')0d0
        write(14,'(f5.1)')tesk(ik)
        goto200
100   enddo
c
200   close(14)
      call output
1005  format(a,f7.2,a)
1010  format(a,f8.5)
1015  format(a,f6.2,2f7.2,f8.5,a)
      end
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine input
      implicit none
c
c common variables
c
      real*8 el1(100),elsk(100),erh(100),te(100),tesk(100)
      integer ntote
      common /ERRS  / te,el1,erh,ntote
      common /ERSK  / tesk,elsk
c
c Local variables
c
      integer i
      real*8 r1,r2
      real*8 dabs
c
      ntote=0
      do while(.true.)
        read(13,*,end=100)r1,r2
        ntote=ntote+1
        te(ntote)=r1
        el1(ntote)=r2
        do i=1,85
          if(dabs(tesk(i)-r1).lt.5d-2)then
            elsk(i)=r2
          endif
        enddo
      enddo
100   return
      end
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine evalgt(mpar)
      implicit none
c
c common variables
c
      real*8 amat(200,200),el1(100),erh(100),te(100)
      integer ntote
      common /ERRS  / te,el1,erh,ntote
      common /MAT   / amat
c
c Dummy arguments
c
      integer mpar
c
c Local variables
c
      real*8 d(100),g(100,3),x
      real*8 dlog
      integer i,j
c
      do i=1,ntote
        x=dlog(te(i))
        g(i,1)=x*x
        g(i,2)=x
        g(i,3)=1d0
        d(i)=el1(i)*el1(i)
      enddo
      do j=1,200
        do i=1,200
          amat(i,j)=0d0
        enddo
      enddo
      do j=1,ntote
        amat(1,j+2*mpar+1)=-1d0
        amat(j+1,1)=d(j)
        amat(j+ntote+1,1)=d(j)
        amat(j+1,j+2*mpar+1)=1d0
        amat(j+ntote+1,j+2*mpar+1)=-1d0
        do i=1,mpar
          amat(j+1,i+1)=g(j,i)
          amat(j+1,i+mpar+1)=-g(j,i)
          amat(j+ntote+1,i+1)=g(j,i)
          amat(j+ntote+1,i+mpar+1)=-g(j,i)
        enddo
      enddo
      return
      end
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine simplx(a,m,n,mp,np,m1,m2,m3,icase,izrov,iposv)
      implicit none
c
c parameter definitions
c
      integer mmax,nmax
      parameter (mmax=200,nmax=200)
      real*8 eps
      parameter (eps=1d-6)
c
c Dummy arguments
c
      integer icase,m,m1,m2,m3,mp,n,np
      real*8 a(mp,np)
      integer iposv(m),izrov(n)
c
c Local variables
c
      real*8 bmax,q1
      integer i,ip,ir,is,k,kh,kp,l1(nmax),l2(mmax),l3(mmax),
     &        m12,nl1,nl2
c
      if(m.ne.m1+m2+m3)pause 'bad input constraint counts in simplx'
      nl1=n
      do k=1,n
        l1(k)=k
        izrov(k)=k
      enddo
      nl2=m
      do i=1,m
        if(a(i+1,1).lt.0d0)pause 'bad input tableau in simplx'
        l2(i)=i
        iposv(i)=n+i
      enddo
      do i=1,m2
        l3(i)=1
      enddo
      ir=0
      if(m2+m3.eq.0)goto400
      ir=1
      do k=1,n+1
        q1=0d0
        do i=m1+1,m
          q1=q1+a(i+1,k)
        enddo
        a(m+2,k)=-q1
      enddo
100   call simp1(a,mp,np,m+1,l1,nl1,0,kp,bmax)
      if(bmax.le.eps.and.a(m+2,1).lt.-eps)then
        icase=-1
        print*,'Aux obj fn is still neg and can"t be improved;'
        print*,'hence no feasible solution exists.'
        return
      elseif(bmax.le.eps.and.a(m+2,1).le.eps)then
        m12=m1+m2+1
        if(m12.le.m)then
          do ip=m12,m
            if(iposv(ip).eq.ip+n)then
              call simp1(a,mp,np,ip,l1,nl1,1,kp,bmax)
              if(bmax.gt.0d0)goto200
            endif
          enddo
        endif
        ir=0
        m12=m12-1
        if(m1+1.le.m12)then
          do i=m1+1,m12
            if(l3(i-m1).eq.1)then
              do k=1,n+1
                a(i+1,k)=-a(i+1,k)
              enddo
            endif
          enddo
        endif
        goto400
      endif
      call simp2(a,m,n,mp,np,l2,nl2,ip,kp,q1)
      if(ip.eq.0)then
        print*,'Max of aux obj fn is unbounded;'
        print*,'hence no feasible solution exists'
        icase=-1
        return
      endif
200   call simp3(a,mp,np,m+1,n,ip,kp)
      if(iposv(ip).ge.n+m1+m2+1)then
        do k=1,nl1
          if(l1(k).eq.kp)goto250
        enddo
250     nl1=nl1-1
        do is=k,nl1
          l1(is)=l1(is+1)
        enddo
      else
        if(iposv(ip).lt.n+m1+1)goto300
        kh=iposv(ip)-m1-n
        if(l3(kh).eq.0)goto300
        l3(kh)=0
      endif
      a(m+2,kp+1)=a(m+2,kp+1)+1d0
      do i=1,m+2
        a(i,kp+1)=-a(i,kp+1)
      enddo
300   is=izrov(kp)
      izrov(kp)=iposv(ip)
      iposv(ip)=is
      if(ir.ne.0)goto100
400   call simp1(a,mp,np,0,l1,nl1,0,kp,bmax)
      if(bmax.le.0d0)then
        icase=0
        return
      endif
      call simp2(a,m,n,mp,np,l2,nl2,ip,kp,q1)
      if(ip.eq.0)then
        print*,'Warning:  Objective fn is unbounded...'
        icase=1
        return
      endif
      call simp3(a,mp,np,m,n,ip,kp)
      goto300
      end
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine simp1(a,mp,np,mm,ll,nll,iabf,kp,bmax)
      implicit none
c
c Dummy arguments
c
      real*8 bmax
      integer iabf,kp,mm,mp,nll,np
      real*8 a(mp,np)
      integer ll(np)
c
c Local variables
c
      integer k
      real*8 test
      real*8 dabs
c
      kp=ll(1)
      bmax=a(mm+1,kp+1)
      do k=2,nll
        if(iabf.eq.0)then
          test=a(mm+1,ll(k)+1)-bmax
        else
          test=dabs(a(mm+1,ll(k)+1))-dabs(bmax)
        endif
        if(test.gt.0d0)then
          bmax=a(mm+1,ll(k)+1)
          kp=ll(k)
        endif
      enddo
      return
      end
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine simp2(a,m,n,mp,np,l2,nl2,ip,kp,q1)
      implicit none
c
c parameter definitions
c
      real*8 eps
      parameter (eps=1d-6)
c
c Dummy arguments
c
      integer ip,kp,m,mp,n,nl2,np
      real*8 q1
      real*8 a(mp,np)
      integer l2(mp)
c
c Local variables
c
      integer i,ii,k
      real*8 q,q0,qp
c
      ip=0
      do i=1,nl2
        if(a(l2(i)+1,kp+1).lt.-eps)goto100
      enddo
      return
100   q1=-a(l2(i)+1,1)/a(l2(i)+1,kp+1)
      ip=l2(i)
      do i=i+1,nl2
        ii=l2(i)
        if(a(ii+1,kp+1).lt.-eps)then
          q=-a(ii+1,1)/a(ii+1,kp+1)
          if(q.lt.q1)then
            ip=ii
            q1=q
          elseif(q.eq.q1)then
            do k=1,n
              qp=-a(ip+1,k+1)/a(ip+1,kp+1)
              q0=-a(ii+1,k+1)/a(ii+1,kp+1)
              if(q0.ne.qp)goto110
            enddo
110         if(q0.lt.qp)ip=ii
          endif
        endif
      enddo
      return
      end
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine simp3(a,mp,np,i1,k1,ip,kp)
      implicit none
c
c Dummy arguments
c
      integer i1,ip,k1,kp,mp,np
      real*8 a(mp,np)
c
c Local variables
c
      integer ii,kk
      real*8 piv
c
      piv=1d0/a(ip+1,kp+1)
      do ii=1,i1+1
        if(ii-1.ne.ip)then
          a(ii,kp+1)=a(ii,kp+1)*piv
          do kk=1,k1+1
            if(kk-1.ne.kp)then
              a(ii,kk)=a(ii,kk)-a(ip+1,kk)*a(ii,kp+1)
            endif
          enddo
        endif
      enddo
      do kk=1,k1+1
        if(kk-1.ne.kp)a(ip+1,kk)=-a(ip+1,kk)*piv
      enddo
      a(ip+1,kp+1)=piv
      return
      end
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine output
      implicit none
c
c common variables
c
      real*8 el1(100),erh(100),te(100)
      integer ntote
      common /ERRS  / te,el1,erh,ntote
c
c Local variables
c
      integer i
c
      rewind(13)
      do i=1,ntote
        write(13,'(f6.2,2f9.6)')te(i),el1(i),erh(i)
      enddo
      return
      end
