c cploads.for **** version: 29 Jul 2004 *******************************
c Deconvolves loading of the lithosphere from Bouguer gravity and
c topography amplitudes given an a priori estimate of elastic
c thickness, a surface fluid density, and a crustal seismic
c velocity profile (the latter is used for regression to density).
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c  Modified by Gareth Armstrong, Dept. Earth Sciences, Uni. Oxford,
c  on 18/2/99 to use the Salford 
c  ftn77 compiler call to COMMAND_LINE to read in command line arguments 
c  similar to the f77 compiler call to getarg when being used on the PC
c  Comment or uncomment relevant calls as required.
c  Changed Units numbers to be above 12 and made array size adjustable
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c********************** main program ********************************
      implicit none
      include 'coheren.inc'
c
c common variables
c
      real*8 alatmn,alatmx,alonmn,alonmx,cd,cnadr(5,2),e,elastt,g,
     &     gc,grad(30),pi,pr,rdl,rho(30),rl(2),ro0,ro1,
     &     rof,spac,theta,vel(30),zdep(30),zl
      complex*16 dhb(ncmax,nrmax),dht(ncmax,nrmax),dwb(ncmax,nrmax),
     &        dwt(ncmax,nrmax)
      integer ilat0,ilon0,nc,nr,nvel
      common /APAR  / nc,nr,rl,spac,ilon0,ilat0,theta,alonmn,
     &                alonmx,alatmn,alatmx
      common /EPAR  / pi,ro0,rdl,ro1,zl,g,gc,e,pr,cd,elastt,rof
      common /LOAD  / dht,dwt,dhb,dwb
      common /VELS  / zdep,vel,grad,rho,cnadr,nvel
c
c Local variables
c
      character card1*20,card2*20,comfile*20,card*40
c
      call getarg(1,comfile)
      call getarg(2,card1)
      call getarg(3,card2)
c
c-----------------------------------------------------------------------
c  THE NEXT call TO COMMAND_LINE PERTAINS TO THE SALFORD FTN77 COMPILER
c  AND IS USED ON THE PC INSTEAD OF f77'S call TO getarg
c-----------------------------------------------------------------------
c
c      call COMMAND_LINE(card)
c      read(card,*)comfile,card1,card2
c   
      read(card1,*)elastt
      elastt=elastt*1d3
      read(card2,*)rof
      open(13,file='modpar.com',form='formatted',status='old')
      rewind(13)
      open(14,file=comfile,form='formatted',status='old')
      rewind(14)
      call input
      call setpar
      cd=e*elastt*elastt*elastt/(12d0*(1d0-pr*pr))
      call twinit
      call output
      end
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine input
      implicit none
      include 'coheren.inc'
c
c common variables
c
      real*8 alatmn,alatmx,alonmn,alonmx,rl(2),spac,theta
      complex*16 dhb(ncmax,nrmax),dht(ncmax,nrmax),dwb(ncmax,nrmax),
     &        dwt(ncmax,nrmax)
      integer ilat0,ilon0,nc,nr
      common /APAR  / nc,nr,rl,spac,ilon0,ilat0,theta,alonmn,
     &                alonmx,alatmn,alatmx
      common /LOAD  / dht,dwt,dhb,dwb
c
c Local variables
c
      character filing*20,filint*20
      integer i,ident,identg,j
c
      read(14,*)filint
      read(14,*)filing
      open(15,file=filint,form='unformatted',status='old')
      rewind(15)
      open(16,file=filing,form='unformatted',status='old')
      rewind(16)
      read(15)ident,ilon0,ilat0,spac,theta
      read(15)nc,nr,rl(2),rl(1)
      read(15)alonmn,alonmx,alatmn,alatmx
      do j=1,nr
        read(15)(dht(i,j),i=1,nc)
      enddo
      read(16)identg,ilon0,ilat0,spac,theta
      read(16)nc,nr,rl(2),rl(1)
      read(16)alonmn,alonmx,alatmn,alatmx
      do j=1,nr
        read(16)(dwt(i,j),i=1,nc)
      enddo
      return
      end
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine setpar
      implicit none
      include 'coheren.inc'
c
c common variables
c
      real*8 cd,cnadr(5,2),e,elastt,g,gc,grad(30),pi,pr,rdl,
     &     rho(30),ro0,ro1,rof,vel(30),zdep(30),zl
      integer nvel
      common /EPAR  / pi,ro0,rdl,ro1,zl,g,gc,e,pr,cd,elastt,rof
      common /VELS  / zdep,vel,grad,rho,cnadr,nvel
c
c Local variables
c
      integer i,nl
      real*8 v2ro
c
      pi=3.14159265358979d0
      g=9.78d0
      gc=6.67d-11
      e=1d11
      pr=0.25d0
      cnadr(1,1)=4929d0
      cnadr(2,1)=5055d0
      cnadr(3,1)=5141d0
      cnadr(4,1)=5212d0
      cnadr(5,1)=5281d0
      cnadr(1,2)=-13294d0
      cnadr(2,2)=-14094d0
      cnadr(3,2)=-14539d0
      cnadr(4,2)=-14863d0
      cnadr(5,2)=-15174d0
      read(13,*)nvel,nl
      zdep(1)=0d0
      read(13,*)vel(1)
      rho(1)=v2ro(vel(1)/1d3,zdep(1)/1d3)
      do i=2,nvel
        read(13,*)zdep(i),vel(i)
        if(vel(i).ne.vel(i-1))then
          rho(i)=v2ro(vel(i)/1d3,zdep(i)/1d3)
        else
          rho(i)=rho(i-1)
        endif
      enddo
      do i=1,nvel-1
        if(zdep(i).ne.zdep(i+1))then
          grad(i)=(rho(i+1)-rho(i))/(zdep(i+1)-zdep(i))
          if(grad(i).lt.2d-3)then
            rho(i)=(rho(i+1)+rho(i))/2d0
            rho(i+1)=rho(i)
            grad(i)=0d0
          endif
        else
          grad(i)=-9999d0
        endif
      enddo
      ro0=rho(1)
      if(zdep(nl).ne.zdep(nl+1))then
        print *,'Load depth is not a first order discontinuity!'
        stop
      endif
      rdl=rho(nl+1)-rho(nl)
      zl=zdep(nl)
      zl=dmin1(zl,5d3)  !  NOTE this version of MECAIR1.1 is unstable to deep loading
      ro1=rho(nvel)
      return
      end
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine twinit
      implicit none
      include 'coheren.inc'
c
c common variables
c
      real*8 alatmn,alatmx,alonmn,alonmx,cd,e,elastt,g,gc,
     &     pi,pr,rdl,rl(2),ro0,ro1,rof,spac,theta,zl
      complex*16 dhb(ncmax,nrmax),dht(ncmax,nrmax),dwb(ncmax,nrmax),
     &        dwt(ncmax,nrmax)
      integer ilat0,ilon0,nc,nr
      common /APAR  / nc,nr,rl,spac,ilon0,ilat0,theta,alonmn,
     &                alonmx,alatmn,alatmx
      common /EPAR  / pi,ro0,rdl,ro1,zl,g,gc,e,pr,cd,elastt,rof
      common /LOAD  / dht,dwt,dhb,dwb
c
c Local variables
c
      real*8 a1,b1,c1,d1
      real*8 aa,ab,ac,ad,ad1,ad2,cgv,cint,dd1,dd2,ep,k2d,k4,kmin,
     &     kzl,pd,rdt,rd1,rd0,smin,td1,td2,tol
      real*8 dfloat
      real*8 cintg
      complex*16 dcmplx
      complex*16 dbot,df,dhi,dtop,dwi,ff
      integer i,j
c
      rdt=ro1-ro0
      rd1=ro1-rof
      rd0=ro0-rof
      cgv=1d5*2d0*pi*gc
      td1=2d0*pi/(spac*dfloat(nc-1)*1d3)
      td2=2d0*pi/(spac*dfloat(nr-1)*1d3)
c Assume a load ratio of -1 when k and the determinant are small:
c df=-1 is empirically found to be optimal for western U.S.; elsewhere can be different
      df=dcmplx(-1d0)
      ff=df*rd0/rdl
c Set determinant tolerance for perturbation of load depth:
      tol=1d-3
c
c Find the wavenumber of minimum for (pd-tol) load depth:
      smin=0d0
      kmin=0d0
      do i=2,10*nc
        k2d=1d-1*td1*dfloat(i-1)
        k4=cd*k2d**4/g
        cint=cintg(k2d,0d0)
        aa=(rdt+k4)/(rd1+k4)
        ab=-rdl/(rd1+k4)
        ac=-cgv*rd0*cint/(rd1+k4)
        dd1=cgv*rdl
        dd2=-dd1*cint/(rd1+k4)
        pd=ab*ac-aa*dd2
        kzl=-dlog((pd+1d-4)/(aa*dd1))/k2d
        if(kzl.ge.smin)then
          smin=kzl
          kmin=k2d
        endif
      enddo
c
c Deconvolve surface and subsurface load contributions:
      do j=1,nr
        if(j.le.nr/2+1)then
          ad2=(td2*dfloat(j-1))**2
        else
          ad2=(td2*dfloat(nr+1-j))**2
        endif
        do i=1,nc
          if(i.le.nc/2+1)then
            ad1=(td1*dfloat(i-1))**2
          else
            ad1=(td1*dfloat(nc+1-i))**2
          endif
          k2d=dsqrt(ad1+ad2)
          k4=cd*k2d**4/g
          dbot=dwt(i,j)
          dtop=dht(i,j)
          cint=cintg(k2d,0d0)
          aa=(rdt+k4)/(rd1+k4)
          ab=-rdl/(rd1+k4)
          ac=-cgv*rd0*cint/(rd1+k4)
          dd1=cgv*rdl
          dd2=-dd1*cint/(rd1+k4)
          pd=ab*ac-aa*dd2
          ep=dexp(-k2d*zl)
          ad=dd1*ep+dd2
          if(k2d.le.kmin.and.dabs(aa*ad-ac*ab).lt.tol)then
            dht(i,j)=aa*(dtop/(aa+ab*ff)+dbot/(ac+ad*ff))/2d0
            dhb(i,j)=ab*(dtop/(ab+aa/ff)+dbot/(ad+ac/ff))/2d0
            dwt(i,j)=ac*(dtop/(aa+ab*ff)+dbot/(ac+ad*ff))/2d0
            dwb(i,j)=ad*(dtop/(ab+aa/ff)+dbot/(ad+ac/ff))/2d0
          else
            if(dabs(aa*ad-ac*ab).gt.tol)then
              kzl=zl
            elseif(k2d.gt.kmin)then
              kzl=-dlog((pd+tol)/(aa*dd1))/k2d
            endif
            ep=dexp(-k2d*kzl)
            ad=dd1*ep+dd2
            a1=1d0/(aa-ab*ac/ad)
            b1=1d0/(ac-aa*ad/ab)
            c1=1d0/(ab-aa*ad/ac)
            d1=1d0/(ad-ab*ac/aa)
            dhi=a1*dtop+b1*dbot
            dwi=c1*dtop+d1*dbot
            dht(i,j)=aa*dhi
            dhb(i,j)=dtop-dht(i,j)
            dwb(i,j)=ad*dwi
            dwt(i,j)=dbot-dwb(i,j)
          endif
        enddo
      enddo
      return
      end
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine output
      implicit none
      include 'coheren.inc'
c
c common variables
c
      real*8 alatmn,alatmx,alonmn,alonmx,rl(2),spac,theta
      complex*16 dhb(ncmax,nrmax),dht(ncmax,nrmax),dwb(ncmax,nrmax),
     &        dwt(ncmax,nrmax)
      integer ilat0,ilon0,nc,nr
      common /APAR  / nc,nr,rl,spac,ilon0,ilat0,theta,alonmn,
     &                alonmx,alatmn,alatmx
      common /LOAD  / dht,dwt,dhb,dwb
c
c Local variables
c
      character filhb*20,filht*20,filwb*20,filwt*20
      integer i,j
c
      read(14,*)filht
      open(17,file=filht,form='unformatted',status='unknown')
      rewind(17)
      write(17)1,ilon0,ilat0,spac,theta
      write(17)nc,nr,rl(2),rl(1)
      write(17)alonmn,alonmx,alatmn,alatmx
      read(14,*)filhb
      open(18,file=filhb,form='unformatted',status='unknown')
      rewind(18)
      write(18)1,ilon0,ilat0,spac,theta
      write(18)nc,nr,rl(2),rl(1)
      write(18)alonmn,alonmx,alatmn,alatmx
      read(14,*)filwt
      open(19,file=filwt,form='unformatted',status='unknown')
      rewind(19)
      write(19)1,ilon0,ilat0,spac,theta
      write(19)nc,nr,rl(2),rl(1)
      write(19)alonmn,alonmx,alatmn,alatmx
      read(14,*)filwb
      open(20,file=filwb,form='unformatted',status='unknown')
      rewind(20)
      write(20)1,ilon0,ilat0,spac,theta
      write(20)nc,nr,rl(2),rl(1)
      write(20)alonmn,alonmx,alatmn,alatmx
      do j=1,nr
        write(17)(dht(i,j),i=1,nc)
        write(18)(dhb(i,j),i=1,nc)
        write(19)(dwt(i,j),i=1,nc)
        write(20)(dwb(i,j),i=1,nc)
      enddo
      return
      end
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      real*8 function v2ro(v,z)
      implicit none
      include 'coheren.inc'
c
c common variables
c
      real*8 cnadr(5,2),grad(30),rho(30),vel(30),zdep(30)
      integer nvel
      common /VELS  / zdep,vel,grad,rho,cnadr,nvel
c
c Dummy arguments
c
      real*8 v,z
c
c Local variables
c
      real*8 a,b
      real*8 dmax1,dmin1
c
      if(z.le.10d0)then
        a=cnadr(1,1)
        b=cnadr(1,2)
      elseif(z.le.20d0)then
        a=(z-10d0)*(cnadr(2,1)-cnadr(1,1))/10d0+cnadr(1,1)
        b=(z-10d0)*(cnadr(2,2)-cnadr(1,2))/10d0+cnadr(1,2)
      elseif(z.le.30d0)then
        a=(z-20d0)*(cnadr(3,1)-cnadr(2,1))/10d0+cnadr(2,1)
        b=(z-20d0)*(cnadr(3,2)-cnadr(2,2))/10d0+cnadr(2,2)
      elseif(z.le.40d0)then
        a=(z-30d0)*(cnadr(4,1)-cnadr(3,1))/10d0+cnadr(3,1)
        b=(z-30d0)*(cnadr(4,2)-cnadr(3,2))/10d0+cnadr(3,2)
      elseif(z.le.50d0)then
        a=(z-40d0)*(cnadr(5,1)-cnadr(4,1))/10d0+cnadr(4,1)
        b=(z-40d0)*(cnadr(5,2)-cnadr(4,2))/10d0+cnadr(4,2)
      else
        a=cnadr(5,1)
        b=cnadr(5,2)
      endif
      v2ro=a+b/v
      v2ro=dmax1(v2ro,2d3)
      v2ro=dmin1(v2ro,3.4d3)
      return
      end
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      real*8 function cintg(k2d,ep)
c*****************************************************************
c Function cintg generates the normalized premultiplier for      *
c   upward continuation of flexure of the 1-d density profile to *
c   a gravity signal at the Earth's surface (but without the     *
c   2*pi*G constant term to convert to gravity. Hence, units are *
c   mks units of density).                                       *
c*****************************************************************
      implicit none
      include 'coheren.inc'
c
c common variables
c
      real*8 cd,cnadr(5,2),e,elastt,g,gc,grad(30),pi,pr,rdl,
     &     rho(30),ro0,ro1,rof,vel(30),zdep(30),zl
      integer nvel
      common /EPAR  / pi,ro0,rdl,ro1,zl,g,gc,e,pr,cd,elastt,rof
      common /VELS  / zdep,vel,grad,rho,cnadr,nvel
c
c Dummy arguments
c
      real*8 ep,k2d
c
c Local variables
c
      integer i
c
      if(k2d.eq.0d0)then
        cintg=ro1-ro0
      else
        cintg=0d0
        do i=1,nvel-1
          if(grad(i).ne.-9999d0)then
c*****************************************************
c Gravity response for a constant velocity gradient: *
c*****************************************************
            cintg=cintg+grad(i)*
     &          (dexp(-k2d*zdep(i))-dexp(-k2d*zdep(i+1)))/k2d
          elseif(zdep(i).eq.zl.and.ep.ne.0d0)then
            cintg=cintg+rdl/ep
          else
            cintg=cintg+(rho(i+1)-rho(i))*dexp(-k2d*zdep(i))
          endif
        enddo
      endif
      return
      end
c
