cccccccc program: memcoh.f ccccc version: 19 Jan 2002 ccccccccccccccccccccccc
c Estimates the radially averaged coherence function from
c maximum entropy auto- and cross-power spectral densities of
c gridded two-dimensional Bouguer gravity and topographic data
c for use in coherence analysis of flexural rigidity.
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c  Modified by Gareth Armstrong,Dept. Earth Sciences,Uni. Oxford,
c  on 17/2/99 to use the Salford 
c  ftn77 compiler call to COMMAND_LINE to read in command line arguments 
c  similar to the f77 compiler call to getarg when being used on the PC
c  Comment or uncomment relevant calls as required.
c  Changed Units numbers to be above 12
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      implicit none
      include 'coheren.inc'
c
c common variables
c
      real*8 adc(31),ak2d(ncmax,nrmax),akc(31),alatmn,alatmx,alonmn,
     &     alonmx,ann(31),c(31),coha(400),errc(400),na(31),
     &     pi,rl(2),spac,theta,wvno(400)
      complex*16 dc2(31),dcs(31),dstr(ncmax,nrmax),dt(ncmax,nrmax),
     &        dw(ncmax,nrmax)
      integer ilat0,ilon0,nac,nc,nr,ntot
      common /ADIM  / nc,nr,rl,ilat0,ilon0,spac,theta,pi,alonmn,
     &                alonmx,alatmn,alatmx
      common /AVRG  / c,adc,akc,na
      common /COH   / wvno,coha,errc,ntot
      common /SPEC  / dc2,dcs,nac
      common /STR   / dstr,dt,dw
      common /WAVE  / ak2d,ann
c
c Local variables
c
      character infile*20,outfile*20,card*40
      integer nnac
c
c
      call getarg(1,infile)
      call getarg(2,outfile)
c
c-----------------------------------------------------------------------
c  THE NEXT call TO COMMAND_LINE PERTAINS TO THE SALFORD FTN77 COMPILER
c  AND IS USED ON THE PC INSTEAD OF f77'S call TO getarg
c-----------------------------------------------------------------------
c
c      call COMMAND_LINE(card)
c      read(card,*)infile,outfile
c
      open(13,file=infile,form='formatted',status='old')
      rewind(13)
      open(14,file=outfile,form='formatted',status='unknown')
      rewind(14)
c
      pi=3.14159265358979d0
c
      call input
c
      call twodk
      ntot=0
      do nnac=20,30
        nac=nnac
        call annbds
        call init
        call avspec
        call delann
        call cohere
        call cram
      enddo
c
      call sortc(wvno,coha,errc,ntot)
      call output
c
      end
c
c--------------------- data input -----------------------------------
      subroutine input
      implicit none
      include 'coheren.inc'
c
c common variables
c
      real*8 alatmn,alatmx,alonmn,alonmx,pi,rl(2),spac,theta
      complex*16 dstr(ncmax,nrmax),dt(ncmax,nrmax),dw(ncmax,nrmax)
      integer ilat0,ilon0,nc,nr
      common /ADIM  / nc,nr,rl,ilat0,ilon0,spac,theta,pi,alonmn,
     &                alonmx,alatmn,alatmx
      common /STR   / dstr,dt,dw
c
c Local variables
c
      character filinc*20,filing*20,filint*20
      integer i,id,j
c
c...  read in data by rows
      read(13,*)filinc
      read(13,*)filing
      read(13,*)filint
      open(15,file=filinc,form='unformatted',status='old')
      rewind(15)
      read(15)id,ilon0,ilat0,spac,theta
      read(15)nc,nr,rl(1),rl(2)
      read(15)alonmn,alonmx,alatmn,alatmx
      do j=1,nr
        read(15)(dstr(i,j),i=1,nc)
      enddo
      open(16,file=filing,form='unformatted',status='old')
      rewind(16)
      read(16)id,ilon0,ilat0,spac,theta
      read(16)nc,nr,rl(1),rl(2)
      read(16)alonmn,alonmx,alatmn,alatmx
      do j=1,nr
        read(16)(dw(i,j),i=1,nc)
      enddo
      open(17,file=filint,form='unformatted',status='old')
      rewind(17)
      read(17)id,ilon0,ilat0,spac,theta
      read(17)nc,nr,rl(1),rl(2)
      read(17)alonmn,alonmx,alatmn,alatmx
      do j=1,nr
        read(17)(dt(i,j),i=1,nc)
      enddo
      return
      end
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine twodk
      implicit none
      include 'coheren.inc'
c
c common variables
c
      real*8 ak2d(ncmax,nrmax),alatmn,alatmx,alonmn,alonmx,
     &     ann(31),pi,rl(2),spac,theta
      integer ilat0,ilon0,nc,nr
      common /ADIM  / nc,nr,rl,ilat0,ilon0,spac,theta,pi,alonmn,
     &                alonmx,alatmn,alatmx
      common /WAVE  / ak2d,ann
c
c Local variables
c
      real*8 ad1,ad2,td1,td2
      real*8 dfloat,dsqrt
      integer i,j
c
      td1=2d0*pi/(spac*dfloat(nc-1))
      td2=2d0*pi/(spac*dfloat(nr-1))
      do j=1,nr/2+1
        ad2=(td2*dfloat(j-1))**2
        do i=1,nc/2+1
          ad1=(td1*dfloat(i-1))**2
          ak2d(i,j)=dsqrt(ad1+ad2)
        enddo
      enddo
      do j=1,nr/2+1
        ad2=(td2*dfloat(j-1))**2
        do i=nc/2+2,nc
          ad1=(td1*dfloat(nc-i+1))**2
          ak2d(i,j)=dsqrt(ad1+ad2)
        enddo
      enddo
      do j=nr/2+2,nr
        ad2=(td2*dfloat(nr-j+1))**2
        do i=1,nc/2+1
          ad1=(td1*dfloat(i-1))**2
          ak2d(i,j)=dsqrt(ad1+ad2)
        enddo
      enddo
      do j=nr/2+2,nr
        ad2=(td2*dfloat(nr-j+1))**2
        do i=nc/2+2,nc
          ad1=(td1*dfloat(nc-i+1))**2
          ak2d(i,j)=dsqrt(ad1+ad2)
        enddo
      enddo
      return
      end
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine annbds
      implicit none
      include 'coheren.inc'
c
c common variables
c
      real*8 ak2d(ncmax,nrmax),alatmn,alatmx,alonmn,alonmx,ann(31),
     &     pi,rl(2),spac,theta
      complex*16 dc2(31),dcs(31)
      integer ilat0,ilon0,nac,nc,nr
      common /ADIM  / nc,nr,rl,ilat0,ilon0,spac,theta,pi,alonmn,
     &                alonmx,alatmn,alatmx
      common /SPEC  / dc2,dcs,nac
      common /WAVE  / ak2d,ann
c
c Local variables
c
      integer i
      real*8 x1,x2,y
      real*8 dmin1
c
      x1=dlog10(dmin1(ak2d(1,2),ak2d(2,1)))
      x2=dlog10(ak2d(nr/2+1,nc/2+1))
      y=(x2-x1)/dfloat(nac)
      do i=1,nac+1
        ann(i)=10d0**(x1+y*dfloat(i-1))
      enddo
      return
      end
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine init
      implicit none
      include 'coheren.inc'
c
c common variables
c
      real*8 adc(31),akc(31),c(31),na(31)
      complex*16 dc2(31),dcs(31)
      integer nac
      common /AVRG  / c,adc,akc,na
      common /SPEC  / dc2,dcs,nac
c
c Local variables
c
      complex*16 dcmplx
      integer i
c
      do i=1,31
        dcs(i)=dcmplx(0d0)
        dc2(i)=dcmplx(0d0)
        c(i)=0d0
        na(i)=0d0
        akc(i)=0d0
      enddo
      return
      end
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine avspec
      implicit none
      include 'coheren.inc'
c
c common variables
c
      real*8 adc(31),ak2d(ncmax,nrmax),akc(31),alatmn,alatmx,alonmn,
     &     alonmx,ann(31),c(31),na(31),pi,rl(2),
     &     spac,theta
      complex*16 dc2(31),dcs(31),dstr(ncmax,nrmax),dt(ncmax,nrmax),
     &     dw(ncmax,nrmax)
      integer ilat0,ilon0,nac,nc,nr
      common /ADIM  / nc,nr,rl,ilat0,ilon0,spac,theta,pi,alonmn,
     &                alonmx,alatmn,alatmx
      common /AVRG  / c,adc,akc,na
      common /SPEC  / dc2,dcs,nac
      common /STR   / dstr,dt,dw
      common /WAVE  / ak2d,ann
c
c Local variables
c
      real*8 cdabs
      complex*16 dc
      integer i,j,l
c
      do j=1,nr
        do i=1,nc
          do l=1,nac
            if((ak2d(i,j).ge.ann(l)).and.(ak2d(i,j).lt.ann(l+1)))then
              if(cdabs(dt(i,j)*dw(i,j)).ne.0d0)then
                dc=dstr(i,j)*dstr(i,j)/(dt(i,j)*dw(i,j))
                if(cdabs(dc).gt.1d0)dc=dc/cdabs(dc)
                dcs(l)=dcs(l)+dc
                dc2(l)=dc2(l)+dc*dc
                na(l)=na(l)+1d0
                akc(l)=akc(l)+ak2d(i,j)
              endif
            endif
          enddo
        enddo
      enddo
      return
      end
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine delann
      implicit none
      include 'coheren.inc'
c
c common variables
c
      real*8 adc(31),ak2d(ncmax,nrmax),akc(31),ann(31),c(31),na(31)
      complex*16 dc2(31),dcs(31)
      integer nac
      common /AVRG  / c,adc,akc,na
      common /SPEC  / dc2,dcs,nac
      common /WAVE  / ak2d,ann
c
c Local variables
c
      integer i,iii,j,nnc
c
100   nnc=nac
      i=1
      iii=2
      do while(i.lt.nac)
        if(na(i).lt.5d0)then
          dcs(i)=dcs(i)+dcs(i+1)
          dc2(i)=dc2(i)+dc2(i+1)
          na(i)=na(i)+na(i+1)
          akc(i)=akc(i)+akc(i+1)
          do j=i+1,nac
            dcs(j)=dcs(j+1)
            dc2(j)=dc2(j+1)
            na(j)=na(j+1)
            akc(j)=akc(j+1)
            ann(j)=ann(j+1)
          enddo
          nnc=nnc-1
          i=nac
          iii=1
        endif
        i=i+1
      enddo
      nac=nnc
      if(iii.eq.1)goto100
      return
      end
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine cohere
      implicit none
      include 'coheren.inc'
c
c common variables
c
      real*8 adc(31),akc(31),c(31),na(31)
      complex*16 dc2(31),dcs(31)
      integer nac
      common /AVRG  / c,adc,akc,na
      common /SPEC  / dc2,dcs,nac
c
c Local variables
c
      real*8 cdabs
      real*8 cb
      integer i
c
      do i=1,nac
        akc(i)=akc(i)/na(i)
        cb=cdabs(dcs(i))/na(i)
        c(i)=cb
        adc(i)=dsqrt(cdabs((na(i)*dc2(i)-dcs(i)**2)/(na(i)*(na(i)-
     &      1d0))))
      enddo
      return
      end
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine cram
      implicit none
      include 'coheren.inc'
c
c common variables
c
      real*8 adc(31),akc(31),c(31),coha(400),errc(400),na(31),
     &     wvno(400)
      complex*16 dc2(31),dcs(31)
      integer nac,ntot
      common /AVRG  / c,adc,akc,na
      common /COH   / wvno,coha,errc,ntot
      common /SPEC  / dc2,dcs,nac
c
c Local variables
c
      integer i,ifl,j
c
      do i=1,nac
        ifl=0
        do j=1,ntot
          if(wvno(j).eq.akc(i))ifl=1
        enddo
        if(ifl.eq.0)then
          ntot=ntot+1
          wvno(ntot)=akc(i)
          coha(ntot)=c(i)
          errc(ntot)=adc(i)
        endif
      enddo
      return
      end
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine sortc(wvn,coh,sdv,nn)
      implicit none
      include 'coheren.inc'
c
c Dummy arguments
c
      integer nn
      real*8 coh(400),sdv(400),wvn(400)
c
c Local variables
c
      integer i,iclnd,isrtd,j
      real*8 tmp
c
      isrtd=0
      do while(isrtd.ne.1)
        isrtd=1
        do i=1,nn-1
          if(wvn(i).gt.wvn(i+1))then
            tmp=wvn(i)
            wvn(i)=wvn(i+1)
            wvn(i+1)=tmp
            tmp=coh(i)
            coh(i)=coh(i+1)
            coh(i+1)=tmp
            tmp=sdv(i)
            sdv(i)=sdv(i+1)
            sdv(i+1)=tmp
            isrtd=0
          endif
        enddo
      enddo
      iclnd=0
      do while(iclnd.ne.1)
        iclnd=1
        do i=1,nn-1
          if(wvn(i).eq.wvn(i+1).or.wvn(i).eq.0d0)then
            do j=i,nn-1
              wvn(j)=wvn(j+1)
              coh(j)=coh(j+1)
              sdv(j)=sdv(j+1)
            enddo
            wvn(nn)=0d0
            coh(nn)=0d0
            sdv(nn)=0d0
            nn=nn-1
            iclnd=0
          endif
        enddo
      enddo
      return
      end
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine output
      implicit none
      include 'coheren.inc'
c
c common variables
c
      real*8 coha(400),errc(400),wvno(400)
      integer ntot
      common /COH   / wvno,coha,errc,ntot
c
c Local variables
c
      integer i
c
      do i=1,ntot
        write(14,1005)wvno(i),coha(i),errc(i)
      enddo
1005  format(g12.5,1x,g12.5,1x,g12.5)
      return
      end

